__package_name__ = 'django-statsd'
__version__ = '2.3.0'
__author__ = 'Rick van Hattem'
__author_email__ = 'Rick.van.Hattem@Fawo.nl'
__description__ = (
    '''django-statsd is a Django app that submits query and view durations '''
    '''to Etsy's statsd.'''
)
__url__ = 'https://github.com/WoLpH/django-statsd'

